@@ -40,16 +40,16 @@ class HypothesisRandom(Random):
         return self.__copy__()
 
     def __repr__(self):
-        raise NotImplementedError()
+        raise NotImplementedError
 
     def seed(self, seed):
-        raise NotImplementedError()
+        raise NotImplementedError
 
     def getstate(self):
-        raise NotImplementedError()
+        raise NotImplementedError
 
     def setstate(self, state):
-        raise NotImplementedError()
+        raise NotImplementedError
 
     def _hypothesis_log_random(self, method, kwargs, result):
         if not (self.__note_method_calls and should_note()):
@@ -62,7 +62,7 @@ class HypothesisRandom(Random):
         report(f"{self!r}.{method}({argstr}) -> {result!r}")
 
     def _hypothesis_do_random(self, method, kwargs):
-        raise NotImplementedError()
+        raise NotImplementedError
 
 
 RANDOM_METHODS = [
@@ -70,6 +70,7 @@ RANDOM_METHODS = [
     for name in [
         "_randbelow",
         "betavariate",
+        "binomialvariate",
         "choice",
         "choices",
         "expovariate",
@@ -96,15 +97,15 @@ RANDOM_METHODS = [
 
 # Fake shims to get a good signature
 def getrandbits(self, n: int) -> int:  # type: ignore
-    raise NotImplementedError()
+    raise NotImplementedError
 
 
 def random(self) -> float:  # type: ignore
-    raise NotImplementedError()
+    raise NotImplementedError
 
 
 def _randbelow(self, n: int) -> int:  # type: ignore
-    raise NotImplementedError()
+    raise NotImplementedError
 
 
 STUBS = {f.__name__: f for f in [getrandbits, random, _randbelow]}
@@ -211,7 +212,7 @@ class ArtificialRandom(HypothesisRandom):
             original = list(seq)
             for i, i2 in enumerate(result):
                 seq[i] = original[i2]
-            return
+            return None
         return result
 
     def _hypothesis_do_random(self, method, kwargs):
@@ -271,6 +272,9 @@ class ArtificialRandom(HypothesisRandom):
                 result = cu.integer_range(self.__data, start, stop - 1)
         elif method == "randint":
             result = cu.integer_range(self.__data, kwargs["a"], kwargs["b"])
+        # New in Python 3.12, so not taken by our coverage job
+        elif method == "binomialvariate":  # pragma: no cover
+            result = cu.integer_range(self.__data, 0, kwargs["n"])
         elif method == "choice":
             seq = kwargs["seq"]
             result = cu.integer_range(self.__data, 0, len(seq) - 1)
@@ -317,8 +321,7 @@ class ArtificialRandom(HypothesisRandom):
             result = self.__data.draw(floats(min_value=0.0))
         elif method == "shuffle":
             result = self.__data.draw(permutations(range(len(kwargs["x"]))))
-        # This is tested for but only appears in 3.9 so doesn't appear in coverage.
-        elif method == "randbytes":  # pragma: no cover
+        elif method == "randbytes":
             n = kwargs["n"]
             result = self.__data.draw(binary(min_size=n, max_size=n))
         else:
@@ -385,7 +388,7 @@ def convert_kwargs(name, kwargs):
         if args[-1] is signature.parameters[name].default:
             args.pop()
         else:
-            break  # pragma: no cover  # Only on Python < 3.8
+            break
 
     return (args, kwargs)
 
